@@ -43,7 +43,7 @@ import org.videolan.vlc.R;
 import org.videolan.vlc.gui.PlaybackServiceFragment;
 import org.videolan.vlc.interfaces.OnEqualizerBarChangeListener;
 import org.videolan.vlc.util.VLCOptions;
-import org.videolan.vlc.widget.EqualizerBar;
+import org.videolan.vlc.gui.view.EqualizerBar;
 
 public class EqualizerFragment extends PlaybackServiceFragment {
 
